@import 'MochaJSDelegate.js'

var settings = {
	identifier: 'co.symu.extension',
	width: 300,
	height: 378,
	titleBar: {
		title: '',
		change: true,
		backgroundColor: NSColor.colorWithRed_green_blue_alpha(0.941, 0.949, 0.961, 1)	
	},
	floatWindow: true,
	dev: false
}

var context;
var windowObject;

function onRun(_context) {

	context = _context;

	COScript.currentCOScript().setShouldKeepAround_(true);
	
	ConfigManagerAdapter.init();
	ScreenShotAdapter.init();
	Helpers.init();

	var Panel = NSPanel.alloc().init();
	Panel.setFrame_display(NSMakeRect(0, 0, settings.width, (settings.height + 22)), false);
	Panel.setTitlebarAppearsTransparent(settings.titleBar.change);
	Panel.setTitle(settings.titleBar.title);
	Panel.standardWindowButton(NSWindowMiniaturizeButton).setHidden(true);
	Panel.standardWindowButton(NSWindowZoomButton).setHidden(true);

	var contentView = Panel.contentView();
	contentView.setWantsLayer(true);
	contentView.layer().setFrame(contentView.frame());
	contentView.layer().setMasksToBounds(true);
	
	var titlebarView = contentView.superview().titlebarViewController().view();
	titlebarView.setBackgroundColor(settings.titleBar.backgroundColor);

	var closeButton = Panel.standardWindowButton(NSWindowCloseButton);
	closeButton.setCOSJSTargetFunction(function(sender) {
		Helpers.deleteTmpFolder();
		Panel.orderOut(nil);
		NSApp.stopModal();
	});
	
	var webView = WebView.alloc().initWithFrame(NSMakeRect(0, 0, settings.width, settings.height));
	windowObject = webView.windowScriptObject();
	contentView.addSubview(webView);
	
	var delegate = new MochaJSDelegate({
		
		"webView:didFinishLoadForFrame:": function(webView, webFrame) {
			
			windowObject.evaluateWebScript("ConfigManager.init('" + ConfigManagerAdapter.read() + "')");
			
			windowObject.evaluateWebScript("Views.upload.loadPreview('" + ScreenShotAdapter.makeThumbnail() + "', '" + Helpers.getName() + "', '" + Helpers.getType() + "')");
			
			windowObject.evaluateWebScript("Views.upload.loadLocation('" + LocationAdapter.get() + "')");
			
			if (!settings.dev) {
				windowObject.evaluateWebScript("document.addEventListener('contextmenu', function(e) { e.preventDefault(); });");
			}
			
		},
		
		"webView:didChangeLocationWithinPageForFrame:": function(webView, webFrame){
		
			var request = NSURL.URLWithString(webView.mainFrameURL()).fragment();
			
			if (request != '') {
								
				var data = JSON.parse(decodeURI(request));

				if (data.action == 'ConfigManagerAdapter.write') {
					
					ConfigManagerAdapter.write(data.options.json);
					
				} else if (data.action == 'Helpers.openLink') {
					
					Helpers.openLink(data.options.link);
					
				} else if (data.action == 'bigWindow') {
					
					Panel.setFrame_display(NSMakeRect(0, 0, settings.width + 20, (settings.height + 22 + 90)), false);
					webView.setFrame(NSMakeRect(0, 0, settings.width + 20, settings.height + 90));
					Panel.center();
					ConfigManagerAdapter.write(data.options.json);
					
				} else if (data.action == 'smallWindow') {
					
					Panel.setFrame_display(NSMakeRect(0, 0, settings.width, (settings.height + 22)), false);
					webView.setFrame(NSMakeRect(0, 0, settings.width, settings.height));
					Panel.center();
					ConfigManagerAdapter.write(data.options.json);
					
				} else if (data.action == 'LocationAdapter.set') {
					
					LocationAdapter.set(data.options.json);
					
				}  else if (data.action == 'ScreenShotAdapter.upload') {
					
					ScreenShotAdapter.upload(data.options.url, data.options.scale);
					
				}

			}
				
		}
		
	});

	var scriptPath = context.scriptPath.stringByDeletingLastPathComponent());
	webView.setFrameLoadDelegate_(delegate.getClassInstance());
	
	if (settings.dev) {
		webView.setMainFrameURL_('http://localhost:8080');	
	} else {
		webView.setMainFrameURL_(scriptPath + '/html/index.html');
	}

	if (settings.floatWindow) {
		Panel.setLevel(NSFloatingWindowLevel);
		Panel.center();
		Panel.makeKeyAndOrderFront(NSApp.mainWindow());
	} else {
		NSApp.runModalForWindow(Panel);
	}
	
	var doc = context.document;
	var page = [doc currentPage];
	var artboard = [page currentArtboard];
	
	var timerDelegate = new MochaJSDelegate({
		
		"update": function() {
			
			var cdoc = context.document;
			
			var cpage = [cdoc currentPage];
			var cartboard = [cpage currentArtboard];
			
			if (doc != cdoc || page != cpage || artboard != cartboard) {
				doc = cdoc;
				page = cpage;
				artboard = cartboard;
				windowObject.evaluateWebScript("Views.upload.loadPreview('" + ScreenShotAdapter.makeThumbnail() + "', '" + Helpers.getName() + "', '" + Helpers.getType() + "')");
				windowObject.evaluateWebScript("Views.upload.loadLocation('" + LocationAdapter.get() + "')");
			} 
			
		}
		
	});
	
	var uiTimer = [NSTimer timerWithTimeInterval:0.5 target:timerDelegate.getClassInstance() selector:"update" userInfo:null repeats:true];      
	[[NSRunLoop mainRunLoop] addTimer:uiTimer forMode:NSRunLoopCommonModes];

};

var ConfigManagerAdapter = {
	
	scriptPath: '',
	path: '',
	fileName: '',
	
	init: function() {
	
		this.scriptPath = context.scriptPath.stringByDeletingLastPathComponent();
		this.path = context.scriptPath.stringByDeletingLastPathComponent().stringByDeletingLastPathComponent().stringByDeletingLastPathComponent().stringByDeletingLastPathComponent());
		this.fileName = settings.identifier;
		
	},
	
	write: function(jsonString) {
		
		var json = [NSString stringWithUTF8String:jsonString];

		[json writeToFile:this.path + '/' + this.fileName + '.json' atomically:true encoding:NSUTF8StringEncoding error:null];
		
	},
	
	read: function() {
		
		var json = [NSString stringWithContentsOfFile:this.path + '/' + this.fileName + '.json' encoding:NSUTF8StringEncoding error:null];
		
		if (json == null) {
			
			var defaultJson = [NSString stringWithContentsOfFile:this.scriptPath + '/config.default.json' encoding:NSUTF8StringEncoding error:null];
			
			[defaultJson writeToFile:this.path + '/' + this.fileName + '.json' atomically:true encoding:NSUTF8StringEncoding error:null];
			
			json = defaultJson;
			
		}
		
		return json;
		
	}
	
}

var ScreenShotAdapter = {
	
	scriptPath: '',
	thumbnailSize: 150,
	delegate: null,
	
	init: function() {
		
		this.scriptPath = context.scriptPath.stringByDeletingLastPathComponent();
		
		log(this.scriptPath);
		
		this.delegate = new MochaJSDelegate({
			
			"connection:didReceiveData:": function(connection, data){
			
				var response = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
				
				log(response);
				
				windowObject.evaluateWebScript("console.log('xd')");
				
				windowObject.evaluateWebScript("Views.upload.uploadDone('" + response + "')");
					
			}
			
		});
		
	},
	
	buildPath: function(name) {
		
		var path = this.scriptPath;
		path += '/tmp/';
		path += name;
		path += '.png';
		
		return path;
		
	},
	
	takeScreenShot: function(scale, thumbnail) {
		
		var doc = context.document;
	
		var page = [doc currentPage];
		var artboard = [page currentArtboard];
		
		var path;
		var rect;
		
		var slice;
		
		if (artboard) {
			
			if (thumbnail) {
				path = this.buildPath('thumbnail');
			} else {
				path = this.buildPath(artboard.name());	
			}
			
			rect = [artboard bounds];//[MSSliceTrimming trimmedRectForLayerAncestry:artboard];
			//slice = [doc exportRequestForArtboardOrSlice:artboard];
			
		} else {
			
			if (thumbnail) {
				path = this.buildPath('thumbnail');
			} else {
				path = this.buildPath(page.name());	
			}
			
	        rect = [page bounds];
	        //slice = [doc exportRequestForArtboardOrSlice:page];
			
		}
		
		//var slice = [MSExportRequest requestWithRect:rect scale:scale];
		doc.saveExportRequest_toFile([doc exportRequestForArtboardOrSlice:artboard], path);
		
		return path;
		
	},
	
	makeThumbnail: function() {
		
		var doc = context.document;
	
		var page = [doc currentPage];
		var artboard = [page currentArtboard];
		
		var rect;
		
		if (artboard) {
			rect = [artboard bounds];
		} else {
	        rect = [page bounds];
		}
		
		var width = rect.size.width;
		var height = rect.size.height;
		var scale = 1;
		
		if (width > height) {
			if (width > (this.thumbnailSize * 2)) {
				scale = this.thumbnailSize * 2 / width;	
			}
		} else {
			if (height > (this.thumbnailSize * 2)) {
				scale = this.thumbnailSize * 2 / height;
			}
		}
		
		return this.takeScreenShot(scale, true);
		
	},
	
	upload: function(url, scale) {
		
		var path = this.takeScreenShot(scale, false);
		var name = [[NSString stringWithString:path] lastPathComponent];
		
		var image = [[NSImage alloc] initWithContentsOfFile:path];
		var imageData = [image TIFFRepresentation];
		var imageRep = [NSBitmapImageRep imageRepWithData:imageData];
		imageData = [imageRep representationUsingType:NSPNGFileType properties:null];

		var request = [[NSMutableURLRequest alloc] init];
		[request setURL:[NSURL URLWithString:url]];
		[request setHTTPMethod:@"POST"];

		var body = [NSMutableData data];
		var boundary = [NSString stringWithFormat:"Random_Boundary_Chars"];
	
		[request addValue:[NSString stringWithFormat:"multipart/form-data; boundary=%@", boundary] forHTTPHeaderField:"Content-Type"];
	
		[body appendData:[[NSString stringWithFormat:"--%@\r\n", boundary] dataUsingEncoding:NSUTF8StringEncoding]];
		[body appendData:[[NSString stringWithString:"Content-Disposition: form-data; name=\"file\"; filename=\"" + name + "\"\r\n"] dataUsingEncoding:NSUTF8StringEncoding]];
		[body appendData:[[NSString stringWithString:"Content-Type: image/png\r\n\r\n"] dataUsingEncoding:NSUTF8StringEncoding]];
		[body appendData:[NSData dataWithData:imageData]];
		[body appendData:[[NSString stringWithString:"\r\n"] dataUsingEncoding:NSUTF8StringEncoding]];
		[body appendData:[[NSString stringWithFormat:"--%@--\r\n", boundary] dataUsingEncoding:NSUTF8StringEncoding]];
	
		[request setHTTPBody:body];
		
		var connect = [[NSURLConnection alloc] initWithRequest:request delegate:this.delegate.getClassInstance()];
		
		return true;
		
	}
	
}

var LocationAdapter = {
	
	get: function() {
		
		var doc = context.document;
	
		var page = [doc currentPage];
		var artboard = [page currentArtboard];

		var command = context.command;

		if (artboard) {
			return [command valueForKey:'location' onLayer:artboard forPluginIdentifier:settings.identifier];
		} else {
	        return [command valueForKey:'location' onLayer:page forPluginIdentifier:settings.identifier];
		}
		
	},
	
	set: function(value) {
		
		var doc = context.document;
	
		var page = [doc currentPage];
		var artboard = [page currentArtboard];

		var command = context.command;

		if (artboard) {
			[command setValue:value forKey:'location' onLayer:artboard forPluginIdentifier:settings.identifier];
		} else {
	        [command setValue:value forKey:'location' onLayer:page forPluginIdentifier:settings.identifier];
		}
		
	}
	
}

var Helpers = {
	
	scriptPath: '',
	
	init: function() {
		
		this.scriptPath = context.scriptPath.stringByDeletingLastPathComponent();
		
	},
	
	openLink: function(link) {
		
		[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:link]];
		
	},
	
	deleteTmpFolder: function() {
		
		var fm = [NSFileManager defaultManager];
		[fm removeItemAtPath:this.scriptPath + '/tmp/' error:null];
		
	},
	
	getName: function() {
		
		var doc = context.document;
	
		var page = [doc currentPage];
		var artboard = [page currentArtboard];
		
		if (artboard) {
			return artboard.name();
		} else {
			return page.name();
		}
		
	},
	
	getType: function() {
		
		var doc = context.document;
	
		var page = [doc currentPage];
		var artboard = [page currentArtboard];
		
		if (artboard) {
			return 'artboard';
		} else {
			return 'page';
		}
		
	}
	
}
